/*
 * Ratiometric Frequency Counter
 *
 * ports.h - hardware definition
 *
 * (c) Axel (XL) Schwenke, axel.schwenke@gmx.net
 *
 * $Id: ports.h 104 2010-12-04 16:19:40Z schwenke $
 */

#ifndef PORTS_H
#define PORTS_H

/*
             ATMEGA8 Pinout
            -----------------
      ISP |Res       ADC5/PC5| Analog
      RS  |PD0            PC4| DB4
      R/W |PD1            PC3| DB3
      E   |PD2            PC2| DB2
      OE  |PD3            PC1| DB1
      FX  |PD4/T0         PC0| DB0
      +5V |Vcc            GND| GND
      GND |GND           ARef| +5V (filtered)
     XTAL |XTAL1         AVcc| +5V (filtered)
     XTAL |XTAL2          SCK| ISP
      DB5 |PD5           MISO| ISP
      DB6 |PD6           MOSI| ISP
      DB7 |PD7            PB2| Clear
     Gate |PB0/ICP1       PB1| Start
           ------------------

DB0 - DB7 .... Data Bus for LCD and 74HC590
RS, R/W, E ... LCD control lines
OE ........... OE/LAT for 74HC590
FX ........... 9th bit f_x Counter (1/2 74HC74)
Gate ......... Gate Output (1/2 74HC74)
Start ........ Gate Control Line
Clear ........ Clear 74HC590 and Gate
ISP .......... to ISP connector
Analog ....... calibration input

*/

#define LCD_CTRL_PORT    PORTD  /* port for control lines   */
#define LCD_RS_PIN       0      /* pin  for RS line         */
#define LCD_RW_PIN       1      /* pin  for RW line         */
#define LCD_E_PIN        2      /* pin  for Enable line     */

#define GATE_PIN         0      /* Port B */
#define START_PIN        1      /* Port B */
#define CLEAR_PIN        2      /* Port B */
#define LAT_OE_PIN       3      /* Port D */
#define FX_PIN           4      /* Port D */
#define ANALOG_PIN       5      /* Port C */

#define lcd_e_high()     LCD_CTRL_PORT |=  _BV(LCD_E_PIN)
#define lcd_e_low()      LCD_CTRL_PORT &= ~_BV(LCD_E_PIN)
#define lcd_rw_high()    LCD_CTRL_PORT |=  _BV(LCD_RW_PIN)
#define lcd_rw_low()     LCD_CTRL_PORT &= ~_BV(LCD_RW_PIN)
#define lcd_rs_high()    LCD_CTRL_PORT |=  _BV(LCD_RS_PIN)
#define lcd_rs_low()     LCD_CTRL_PORT &= ~_BV(LCD_RS_PIN)

#define clear_high()     PORTB |= _BV(CLEAR_PIN)
#define clear_low()      PORTB &= ~_BV(CLEAR_PIN)
#define start_high()     PORTB |= _BV(START_PIN)
#define start_low()      PORTB &= ~_BV(START_PIN)
#define lat_oe_high()    PORTD |=  _BV(LAT_OE_PIN)
#define lat_oe_low()     PORTD &= ~_BV(LAT_OE_PIN)

#define gate_is_open()   ((PINB & _BV(GATE_PIN))==0)
#define gate_is_closed() (PINB & _BV(GATE_PIN))
#define fx9_read()       ((PIND >> FX_PIN) & 1)

#define data_output()    DDRC |=  0x1F; DDRD |=  0xE0
#define data_input()     DDRC &= ~0x1F; DDRD &= ~0xE0
#define data_write(x)    PORTC = (PORTC & ~0x1F) | ((x) & 0x1F); \
                         PORTD = (PORTD & ~0xE0) | ((x) & 0xE0)
#define data_read()      ((PINC & 0x1F) | (PIND & 0xE0))

#endif
